@@ -1,5 +1,4 @@
-"""
-Wraps individual functions in openjp2 library.
+"""Wraps individual functions in openjp2 library.
 """
 
 # Standard library imports
@@ -17,8 +16,7 @@ OPENJP2 = glymur_config('openjp2')
 
 
 class OpenJPEGLibraryError(IOError):
-    """
-    Issue when the OpenJPEG library signals an error.
+    """Issue when the OpenJPEG library signals an error.
     """
     pass
 
@@ -76,7 +74,7 @@ CODEC_JP2 = 2
 
 
 class PocType(ctypes.Structure):
-    """Progression order changes.
+    """Progression order changes
 
     Corresponds to poc_t type in openjp2 headers.
     """
@@ -617,8 +615,7 @@ def create_compress(codec_format):
 
 
 def codec_set_threads(codec, num_threads):
-    """
-    Allocates worker threads for the compressor/decompressor.
+    """Allocates worker threads for the compressor/decompressor.
 
     This function Wraps the openjp2 library function opj_codec_set_threads.
 
@@ -846,7 +843,8 @@ def end_decompress(codec, stream):
 
 
 def get_num_cpus():
-    """
+    """Retrieve number  of CPUs.
+
     Parameters
     ----------
     None
@@ -860,8 +858,7 @@ def get_num_cpus():
 
 
 def has_thread_support():
-    """
-    Is the library configured with thread support?
+    """Is the library configured with thread support?
 
     Returns
     -------
@@ -953,7 +950,7 @@ def image_tile_create(comptparms, clrspc):
     return image
 
 
-def encoder_set_extra_options(codec, plt=False):
+def encoder_set_extra_options(codec, plt=False, tlm=False):
     """Specify extra options for the encoder.
 
     Wraps the openjp2 library function opj_encoder_set_extra_options.
@@ -963,8 +960,10 @@ def encoder_set_extra_options(codec, plt=False):
     codec:  codec_t
         The JPEG2000 codec to read.
     plt : bool
-        If True, PLT marker segments indicating the length of each packet in
-        the tile-part header will be written.
+        If True, PLT segments indicating the length of each packet in the
+        tile-part header will be written.
+    tlm : bool
+        If True, TLM segments will be generated.
     """
     ARGTYPES = [
         CODEC_TYPE,
@@ -973,9 +972,13 @@ def encoder_set_extra_options(codec, plt=False):
     OPENJP2.opj_encoder_set_extra_options.argtypes = ARGTYPES
     OPENJP2.opj_encoder_set_extra_options.restype = check_error
 
-    arr = (ctypes.c_char_p * 2)()
+    arr = (ctypes.c_char_p * 3)()
     arr[0] = 'PLT=YES'.encode('utf-8') if plt else 'PLT=NO'.encode('utf-8')
-    arr[1] = None
+    if version() >= '2.5.0':
+        arr[1] = 'TLM=YES'.encode('utf-8') if tlm else 'TLM=NO'.encode('utf-8')
+        arr[2] = None
+    else:
+        arr[1] = None
 
     OPENJP2.opj_encoder_set_extra_options(codec, arr)
 
