@@ -1,16 +1,20 @@
 # standard library imports
+from __future__ import annotations
 import io
 import logging
+import pathlib
 import shutil
 import struct
+import sys
+from typing import List, Tuple
+from uuid import UUID
 import warnings
 
 # 3rd party library imports
 import numpy as np
-from uuid import UUID
 
 # local imports
-from glymur import Jp2k
+from glymur import Jp2k, set_option
 from glymur.core import SRGB, RESTRICTED_ICC_PROFILE
 from .lib import tiff as libtiff
 from .lib.tiff import DATATYPE2FMT
@@ -31,20 +35,32 @@ class Tiff2Jp2k(object):
 
     Attributes
     ----------
+    create_exif_uuid : bool
+        Create a UUIDBox for the TIFF IFD metadata.
+    dtype : np.dtype
+        Datatype of the image.
     found_geotiff_tags : bool
         If true, then this TIFF must be a GEOTIFF
-    tiff_filename : path or str
-        Path to TIFF file.
+    imageheight, imagewidth : int
+        Dimensions of the image.
     jp2 : JP2K object
         Write to this JPEG2000 file
-    jp2_filename : path or str
+    jp2_filename : path
         Path to JPEG 2000 file to be written.
     jp2_kwargs : dict
         Keyword arguments to pass along to the Jp2k constructor.
+    photo : int
+        The photometric interpretation of the image.
+    rps : int
+        The number of rows per strip in the TIFF.
+    spp : int
+        Samples Per Pixel TIFF tag value
+    tiff_filename : path
+        Path to TIFF file.
     tilesize : tuple
         The dimensions of a tile in the JP2K file.
-    create_exif_uuid : bool
-        Create a UUIDBox for the TIFF IFD metadata.
+    tw, th : int
+        The tile dimensions for the TIFF image.
     version : int
         Identifies the TIFF as 32-bit TIFF or 64-bit TIFF.
     xmp_data : bytes
@@ -52,10 +68,16 @@ class Tiff2Jp2k(object):
     """
 
     def __init__(
-        self, tiff_filename, jp2_filename,
-        create_exif_uuid=True, create_xmp_uuid=True,
-        exclude_tags=None, tilesize=None,
-        verbosity=logging.CRITICAL, include_icc_profile=False,
+        self,
+        tiff_filename: pathlib.Path,
+        jp2_filename: pathlib.Path,
+        create_exif_uuid: bool = True,
+        create_xmp_uuid: bool = True,
+        exclude_tags: List[int | str] | None = None,
+        num_threads: int = 1,
+        tilesize: Tuple[int, int] | None = None,
+        include_icc_profile: bool = False,
+        verbosity: int = logging.CRITICAL,
         **kwargs
     ):
         """
@@ -112,6 +134,9 @@ class Tiff2Jp2k(object):
 
         self.setup_logging(verbosity)
 
+        if num_threads > 1:
+            set_option('lib.num_threads', num_threads)
+
     def _process_exclude_tags(self, exclude_tags):
         """The list of tags to exclude may be mixed type (str or integer).
         There is also the possibility that they may be capitalized differently
@@ -120,16 +145,13 @@ class Tiff2Jp2k(object):
 
         Parameters
         ----------
-        exclude_tags : list or None
+        exclude_tags : list
             List of tags that are meant to be excluded from the EXIF UUID.
 
         Returns
         -------
         list of numeric tag values
         """
-        if exclude_tags is None:
-            return exclude_tags
-
         lst = []
 
         # first, make the tags all str datatype
@@ -579,13 +601,14 @@ class Tiff2Jp2k(object):
         elif data[0] == 77 and data[1] == 77:
             # big endian
             self.endian = '>'
-        else:
-            msg = (
-                f"The byte order indication in the TIFF header "
-                f"({data}) is invalid.  It should be either "
-                f"{bytes([73, 73])} or {bytes([77, 77])}."
-            )
-            raise RuntimeError(msg)
+        # no other option is possible, libtiff.open would have errored out
+        # else:
+        #     msg = (
+        #         f"The byte order indication in the TIFF header "
+        #         f"({data}) is invalid.  It should be either "
+        #         f"{bytes([73, 73])} or {bytes([77, 77])}."
+        #     )
+        #     raise RuntimeError(msg)
 
         # version number and offset to the first IFD
         version, = struct.unpack(self.endian + 'H', buffer[2:4])
@@ -628,10 +651,10 @@ class Tiff2Jp2k(object):
         else:
             isTiled = False
 
-        photo = self.get_tag_value(262)
-        imagewidth = self.get_tag_value(256)
-        imageheight = self.get_tag_value(257)
-        spp = self.get_tag_value(277)
+        self.photo = self.get_tag_value(262)
+        self.imagewidth = self.get_tag_value(256)
+        self.imageheight = self.get_tag_value(257)
+        self.spp = self.get_tag_value(277)
         sf = self.get_tag_value(339)
         bps = self.get_tag_value(258)
         planar = self.get_tag_value(284)
@@ -645,18 +668,18 @@ class Tiff2Jp2k(object):
             )
             raise RuntimeError(msg)
 
-        if bps not in [8, 16]:
+        if bps == 8 and sf == libtiff.SampleFormat.UINT:
+            self.dtype = np.uint8
+        elif bps == 16 and sf == libtiff.SampleFormat.UINT:
+            self.dtype = np.uint16
+        else:
             msg = (
-                f"The TIFF BitsPerSample is {bps}.  Only 8 and 16 bits per "
-                "sample are supported."
+                f"Only unsigned sample formats and bits per sample values of "
+                f"8 or 16 are supported.  The values in the TIFF are {bps} "
+                f"{sf}."
             )
             raise RuntimeError(msg)
 
-        if bps == 8 and sf == libtiff.SampleFormat.UINT:
-            dtype = np.uint8
-        if bps == 16 and sf == libtiff.SampleFormat.UINT:
-            dtype = np.uint16
-
         if (
             planar == libtiff.PlanarConfig.SEPARATE
             and self.tilesize is not None
@@ -668,70 +691,40 @@ class Tiff2Jp2k(object):
             raise RuntimeError(msg)
 
         if libtiff.isTiled(self.tiff_fp):
-            tw = self.get_tag_value(322)
-            th = self.get_tag_value(323)
+            self.tw = self.get_tag_value(322)
+            self.th = self.get_tag_value(323)
         else:
-            tw = imagewidth
-            rps = self.get_tag_value(278)
-
-        if self.tilesize is not None:
-            # The JP2K tile size was specified.  Compute the number of JP2K
-            # tile rows and columns.
-            jth, jtw = self.tilesize
-
-            num_jp2k_tile_rows = int(np.ceil(imagewidth / jtw))
-            num_jp2k_tile_cols = int(np.ceil(imagewidth / jtw))
-
-        if photo == libtiff.Photometric.YCBCR:
-            # Using the RGBA interface is the only reasonable way to deal with
-            # this.
-            use_rgba_interface = True
-        else:
-            use_rgba_interface = False
+            self.tw = self.imagewidth
+            self.rps = self.get_tag_value(278)
 
         self.jp2 = Jp2k(
             self.jp2_filename,
-            shape=(imageheight, imagewidth, spp),
+            shape=(self.imageheight, self.imagewidth, self.spp),
             tilesize=self.tilesize,
             **self.jp2_kwargs
         )
 
         if not libtiff.RGBAImageOK(self.tiff_fp):
-            photometric_string = self.tagvalue2str(libtiff.Photometric, photo)
+            photometric_string = self.tagvalue2str(
+                libtiff.Photometric, self.photo
+            )
             msg = (
                 f"The TIFF Photometric tag is {photometric_string}.  It is "
                 "not supported by this program."
             )
             raise RuntimeError(msg)
-
-        elif self.tilesize is None:
-
-            # this handles both cases of a striped TIFF and a tiled TIFF
-
-            self._write_rgba_single_tile(
-                    photo, imagewidth, imageheight, spp
-            )
+        elif self.tilesize is None and self.photo == libtiff.Photometric.YCBCR:
+            # this handles both YCbCr cases of a striped TIFF and a tiled TIFF
+            self._write_rgba_single_tile()
             self.jp2.finalize(force_parse=True)
-
+        elif self.tilesize is None and isTiled:
+            self._write_tiled_tiff_to_single_tile_jp2k()
+        elif self.tilesize is None and not isTiled:
+            self._write_stripped_tiff_to_single_tile_jp2k()
         elif isTiled and self.tilesize is not None:
-
-            self._write_tiled_tiff_to_tiled_jp2k(
-                imagewidth, imageheight, spp,
-                jtw, jth, tw, th,
-                num_jp2k_tile_cols, num_jp2k_tile_rows,
-                dtype,
-                use_rgba_interface
-            )
-
+            self._write_tiled_tiff_to_tiled_jp2k()
         elif not isTiled and self.tilesize is not None:
-
-            self._write_striped_tiff_to_tiled_jp2k(
-                imagewidth, imageheight, spp,
-                jtw, jth, rps,
-                num_jp2k_tile_cols, num_jp2k_tile_rows,
-                dtype,
-                use_rgba_interface
-            )
+            self._write_striped_tiff_to_tiled_jp2k()
 
     def tagvalue2str(self, cls, tag_value):
         """Take a class that encompasses all of a tag's allowed values and find
@@ -744,19 +737,11 @@ class Tiff2Jp2k(object):
 
         return tag_value_string
 
-    def _write_rgba_single_tile(
-        self, photo, imagewidth, imageheight, spp
-    ):
+    def _write_rgba_single_tile(self):
         """If no jp2k tiling was specified and if the image is ok to read
         via the RGBA interface, then just do that.  The image will be
         written with the tilesize equal to the image size, so it will
         be written using a single write operation.
-
-        Parameters
-        ----------
-        photo, imagewidth, imageheight, spp : int
-            TIFF tag values corresponding to the photometric interpretation,
-            image width and height, and samples per pixel
         """
         msg = (
             "Reading using the RGBA interface, writing as a single tile "
@@ -764,268 +749,335 @@ class Tiff2Jp2k(object):
         )
         self.logger.info(msg)
 
-        if photo not in [
+        if self.photo not in [
             libtiff.Photometric.MINISWHITE,
             libtiff.Photometric.MINISBLACK,
             libtiff.Photometric.PALETTE,
             libtiff.Photometric.YCBCR,
             libtiff.Photometric.RGB
         ]:
-            photostr = self.tagvalue2str(libtiff.Photometric, photo)
+            photostr = self.tagvalue2str(libtiff.Photometric, self.photo)
             msg = (
-                "Beware, the RGBA interface to attempt to read this TIFF "
+                "Beware, the RGBA interface is attempting to read this TIFF "
                 f"when it has a PhotometricInterpretation of {photostr}."
             )
             warnings.warn(msg)
 
         image = libtiff.readRGBAImageOriented(
-            self.tiff_fp, imagewidth, imageheight
+            self.tiff_fp, self.imagewidth, self.imageheight
         )
 
-        if spp < 4:
+        # must reorder image planes on big-endian
+        if sys.byteorder == 'big':
+            image = np.flip(image, axis=2)
+
+        # potentially get rid of the alpha plane
+        if self.spp < 4:
             image = image[:, :, :3]
 
         self.jp2[:] = image
 
-    def _write_tiled_tiff_to_tiled_jp2k(
-        self, imagewidth, imageheight, spp, jtw, jth, tw, th,
-        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface
-    ):
+    def _write_stripped_tiff_to_single_tile_jp2k(self):
+        """The input TIFF image is stripped and we are to create the output
+        JPEG2000 image as a single tile.
+        """
+        num_tiff_strip_rows = int(np.ceil(self.imageheight / self.rps))
+
+        # This might be a bit bigger than the actual image because of a
+        # possibly partial last strip.
+        stripped_shape = (
+            num_tiff_strip_rows * self.rps, self.imagewidth, self.spp
+        )
+        image = np.zeros(stripped_shape, dtype=self.dtype)
+
+        tiff_strip = np.zeros(
+            (self.rps, self.imagewidth, self.spp), dtype=self.dtype
+        )
+
+        # manually collect all the tiff strips, stuff into the image
+        for stripnum in range(num_tiff_strip_rows):
+            rows = slice(stripnum * self.rps, (stripnum + 1) * self.rps)
+            libtiff.readEncodedStrip(self.tiff_fp, stripnum, tiff_strip)
+            image[rows, :, :] = tiff_strip
+
+        if self.imageheight != stripped_shape[0]:
+            # cut the image down due to a partial last strip
+            image = image[:self.imageheight, :, :]
+
+        self.jp2[:] = image
+
+    def _write_tiled_tiff_to_single_tile_jp2k(self):
+        """The input TIFF image is tiled and we are to create the output
+        JPEG2000 image as a single tile.
+        """
+        num_tiff_tile_cols = int(np.ceil(self.imagewidth / self.tw))
+        num_tiff_tile_rows = int(np.ceil(self.imageheight / self.th))
+
+        # tiled shape might differ from the final image shape if we have
+        # partial tiles on the bottom and on the right
+        final_shape = self.imageheight, self.imagewidth, self.spp
+        tiled_shape = (
+            num_tiff_tile_rows * self.th,
+            num_tiff_tile_cols * self.tw,
+            self.spp
+        )
+
+        image = np.zeros(tiled_shape, dtype=self.dtype)
+        tiff_tile = np.zeros((self.th, self.tw, self.spp), dtype=self.dtype)
+
+        # manually collect all the tiff tiles, stuff into the image
+        for tr in range(num_tiff_tile_rows):
+
+            rows = slice(tr * self.th, (tr + 1) * self.th)
+
+            for tc in range(num_tiff_tile_cols):
+                ttile_num = num_tiff_tile_cols * tr + tc
+                libtiff.readEncodedTile(self.tiff_fp, ttile_num, tiff_tile)
+
+                cols = slice(tc * self.tw, (tc + 1) * self.tw)
+
+                image[rows, cols, :] = tiff_tile
+
+        if final_shape != tiled_shape:
+            image = image[:final_shape[0], :final_shape[1], :]
+
+        self.jp2[:] = image
+
+    def _write_tiled_tiff_to_tiled_jp2k(self):
         """The input TIFF image is tiled and we are to create the output
         JPEG2000 image with specific tile dimensions.
-
-        Parameters
-        ----------
-        imagewidth, imageheight, spp : int
-            TIFF tag values corresponding to the photometric interpretation,
-            image width and height, and samples per pixel.
-        jtw, jth : int
-            The tile dimensions for the JPEG2000 image.
-        tw, th : int
-            The tile dimensions for the TIFF image.
-        num_jp2k_tile_cols, num_jp2k_tile_rows
-            The number of tiles down the rows and across the columns for the
-            JPEG2000 image
-        dtype : np.dtype
-            Datatype of the image.
-        use_rgba_interface : bool
-            If true, use the RGBA interface to read the TIFF image data.
         """
-
-        num_tiff_tile_cols = int(np.ceil(imagewidth / tw))
-
-        partial_jp2_tile_rows = (imageheight / jth) != (imageheight // jth)
-        partial_jp2_tile_cols = (imagewidth / jtw) != (imagewidth // jtw)
-
-        rgba_tile = np.zeros((th, tw, 4), dtype=np.uint8)
-
-        self.logger.debug(f'image:  {imageheight} x {imagewidth}')
-        self.logger.debug(f'jptile:  {jth} x {jtw}')
-        self.logger.debug(f'ttile:  {th} x {tw}')
-        for idx, tilewriter in enumerate(self.jp2.get_tilewriters()):
-
-            # populate the jp2k tile with tiff tiles
-            msg = (
-                f'Tile:  #{idx} '
-                f'row #{idx // num_jp2k_tile_cols} '
-                f'col #{idx % num_jp2k_tile_cols} '
-            )
-            self.logger.info(msg)
-
-            self.logger.debug(f'J tile row:  #{idx // num_jp2k_tile_cols}')
-            self.logger.debug(f'J tile col:  #{idx % num_jp2k_tile_cols}')
-
-            jp2k_tile = np.zeros((jth, jtw, spp), dtype=dtype)
-            tiff_tile = np.zeros((th, tw, spp), dtype=dtype)
-
-            jp2k_tile_row = int(np.ceil(idx // num_jp2k_tile_cols))
-            jp2k_tile_col = int(np.ceil(idx % num_jp2k_tile_cols))
-
-            # the coordinates of the upper left pixel of the jp2k tile
-            julr, julc = jp2k_tile_row * jth, jp2k_tile_col * jtw
-
-            # loop while the upper left corner of the current tiff file is
-            # less than the lower left corner of the jp2k tile
-            r = julr
-            while (r // th) * th < min(julr + jth, imageheight):
-                c = julc
-
-                tilenum = libtiff.computeTile(self.tiff_fp, c, r, 0, 0)
-                self.logger.debug(f'TIFF tile # {tilenum}')
-
-                tiff_tile_row = int(np.ceil(tilenum // num_tiff_tile_cols))
-                tiff_tile_col = int(np.ceil(tilenum % num_tiff_tile_cols))
-
-                # the coordinates of the upper left pixel of the TIFF tile
-                tulr = tiff_tile_row * th
-                tulc = tiff_tile_col * tw
-
-                # loop while the left corner of the current tiff tile is
-                # less than the right hand corner of the jp2k tile
-                while ((c // tw) * tw) < min(julc + jtw, imagewidth):
-
-                    if use_rgba_interface:
-                        libtiff.readRGBATile(
-                            self.tiff_fp, tulc, tulr, rgba_tile
-                        )
-
-                        # flip the tile upside down!!
-                        tiff_tile = np.flipud(rgba_tile[:, :, :3])
-                    else:
-                        libtiff.readEncodedTile(
-                            self.tiff_fp, tilenum, tiff_tile
-                        )
-
-                    # determine how to fit this tiff tile into the jp2k
-                    # tile
-                    #
-                    # these are the section coordinates in image space
-                    ulr = max(julr, tulr)
-                    llr = min(julr + jth, tulr + th)
-
-                    ulc = max(julc, tulc)
-                    urc = min(julc + jtw, tulc + tw)
-
-                    # convert to JP2K tile coordinates
-                    jrows = slice(ulr % jth, (llr - 1) % jth + 1)
-                    jcols = slice(ulc % jtw, (urc - 1) % jtw + 1)
-
-                    # convert to TIFF tile coordinates
-                    trows = slice(ulr % th, (llr - 1) % th + 1)
-                    tcols = slice(ulc % tw, (urc - 1) % tw + 1)
-
-                    jp2k_tile[jrows, jcols, :] = tiff_tile[trows, tcols, :]
-
-                    # move exactly one tiff tile over
-                    c += tw
-
-                    tilenum = libtiff.computeTile(self.tiff_fp, c, r, 0, 0)
-
-                    tiff_tile_row = int(
-                        np.ceil(tilenum // num_tiff_tile_cols)
-                    )
-                    tiff_tile_col = int(
-                        np.ceil(tilenum % num_tiff_tile_cols)
-                    )
-
-                    # the coordinates of the upper left pixel of the TIFF
-                    # tile
-                    tulr = tiff_tile_row * th
-                    tulc = tiff_tile_col * tw
-
-                r += th
-
-            # last tile column?  If so, we may have a partial tile.
-            if (
-                partial_jp2_tile_cols
-                and jp2k_tile_col == num_jp2k_tile_cols - 1
-            ):
-                last_j2k_cols = slice(0, imagewidth - julc)
-                jp2k_tile = jp2k_tile[:, last_j2k_cols, :].copy()
-            if (
-                partial_jp2_tile_rows
-                and jp2k_tile_row == num_jp2k_tile_rows - 1
-            ):
-                last_j2k_rows = slice(0, imageheight - julr)
-                jp2k_tile = jp2k_tile[last_j2k_rows, :, :].copy()
-
+        for jp2k_tilenum, tilewriter in enumerate(self.jp2.get_tilewriters()):
+            tiff_tiles = self._get_covering_tiles(jp2k_tilenum)
+            jp2k_tile = self._cover_tile(jp2k_tilenum, tiff_tiles)
+            self.logger.info(f'Writing tile {jp2k_tilenum}')
             tilewriter[:] = jp2k_tile
 
-    def _write_striped_tiff_to_tiled_jp2k(
-        self, imagewidth, imageheight, spp, jtw, jth, rps,
-        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface
-    ):
-        """The input TIFF image is striped and we are to create the output
-        JPEG2000 image as a tiled JP2K.
+    def _cover_tile(self, jp2k_tile_num, tiff_tile_nums):
+        """
+        Fill in the jp2k tile with image data from the TIFF tiles.
 
         Parameters
         ----------
-        imagewidth, imageheight, spp : int
-            TIFF tag values corresponding to the photometric interpretation,
-            image width and height, and samples per pixel.
-        jtw, jth : int
-            The tile dimensions for the JPEG2000 image.
-        rps : int
-            The number of rows per strip in the TIFF.
-        num_jp2k_tile_cols, num_jp2k_tile_rows
-            The number of tiles down the rows and across the columns for the
-            JPEG2000 image
-        dtype : np.dtype
-            Datatype of the image.
-        use_rgba_interface : bool
-            If true, use the RGBA interface to read the TIFF image data.
+        jp2k_tilenum : int
+            number of the JPEG2000 tile
+        tiff_tile_nums : set
+            all TIFF tile identifiers that cover the jpeg2000 tile
+        """
+        jth, jtw = self.tilesize
+
+        # Does the JP2K have partial tiles on the far right and bottom of the
+        # image.
+        partial_jp2_tile_rows = (self.imageheight / jth) != (self.imageheight // jth)  # noqa : E501
+        partial_jp2_tile_cols = (self.imagewidth / jtw) != (self.imagewidth // jtw)  # noqa : E501
+
+        num_jp2k_tile_rows = int(np.ceil(self.imagewidth / jtw))
+        num_jp2k_tile_cols = int(np.ceil(self.imagewidth / jtw))
+
+        jp2k_tile_row = int(np.ceil(jp2k_tile_num // num_jp2k_tile_cols))
+        jp2k_tile_col = int(np.ceil(jp2k_tile_num % num_jp2k_tile_cols))
+
+        num_tiff_tile_cols = int(np.ceil(self.imagewidth / self.tw))
+
+        jp2k_tile = np.zeros((jth, jtw, self.spp), dtype=self.dtype)
+
+        # coordinates of jp2k upper left corner
+        jp2k_ulx = jtw * jp2k_tile_col
+        jp2k_uly = jth * jp2k_tile_row
+
+        for ttile_num in tiff_tile_nums:
+
+            # compute the coordinates of the upper left pixel of the TIFF tile
+            tiff_tile_row = int(np.ceil(ttile_num // num_tiff_tile_cols))
+            tiff_tile_col = int(np.ceil(ttile_num % num_tiff_tile_cols))
+            tile_uly = tiff_tile_row * self.th
+            tile_ulx = tiff_tile_col * self.tw
+
+            if self.photo == libtiff.Photometric.YCBCR:
+
+                rgba_tile = np.zeros((self.th, self.tw, 4), dtype=np.uint8)
+
+                tiff_tile_row = int(np.ceil(ttile_num // num_tiff_tile_cols))
+                tiff_tile_col = int(np.ceil(ttile_num % num_tiff_tile_cols))
+                x = tiff_tile_col * self.tw
+                y = tiff_tile_row * self.th
+
+                libtiff.readRGBATile(self.tiff_fp, x, y, rgba_tile)
+
+                # The RGBA interface requires some reordering.
+                if sys.byteorder == 'little':
+                    # image is upside down
+                    dims = [0]
+                else:
+                    # image is upside down, but in addition,
+                    # if big-endian, must also flip the image planes
+                    dims = [0, 2]
+                rgba_tile = np.flip(rgba_tile, axis=dims)
+
+                # We may need to remove the alpha plane.
+                tiff_tile = rgba_tile[:, :, :3]
+
+            else:
+
+                tiff_tile = np.zeros(
+                    (self.th, self.tw, self.spp), dtype=self.dtype
+                )
+                libtiff.readEncodedTile(self.tiff_fp, ttile_num, tiff_tile)
+
+            # determine how to fit this tiff tile into the jp2k
+            # tile
+            #
+            # these are the section coordinates in image space
+            uly = max(jp2k_uly, tile_uly)
+            lly = min(jp2k_uly + jth, tile_uly + self.th)
+
+            ulx = max(jp2k_ulx, tile_ulx)
+            urx = min(jp2k_ulx + jtw, tile_ulx + self.tw)
+
+            # convert to JP2K tile coordinates
+            jrows = slice(uly % jth, (lly - 1) % jth + 1)
+            jcols = slice(ulx % jtw, (urx - 1) % jtw + 1)
+
+            # convert to TIFF tile coordinates
+            trows = slice(uly % self.th, (lly - 1) % self.th + 1)
+            tcols = slice(ulx % self.tw, (urx - 1) % self.tw + 1)
+
+            jp2k_tile[jrows, jcols, :] = tiff_tile[trows, tcols, :]
+
+        # last tile column?  last tile row?  If so, we may have a partial tile.
+        if (
+            partial_jp2_tile_cols
+            and jp2k_tile_col == num_jp2k_tile_cols - 1
+        ):
+            last_j2k_cols = slice(0, self.imagewidth - jp2k_ulx)
+            jp2k_tile = jp2k_tile[:, last_j2k_cols, :].copy()
+
+        if (
+            partial_jp2_tile_rows
+            and jp2k_tile_row == num_jp2k_tile_rows - 1
+        ):
+            last_j2k_rows = slice(0, self.imageheight - jp2k_uly)
+            jp2k_tile = jp2k_tile[last_j2k_rows, :, :].copy()
+
+        return jp2k_tile
+
+    def _get_covering_tiles(self, jp2k_tile_num):
+        """
+        Construct the set of TIFF tiles that completely cover the jpeg2000
+        tile.
+        """
+        jth, jtw = self.tilesize
+
+        num_jp2k_tile_cols = int(np.ceil(self.imagewidth / jtw))
+
+        jp2k_tile_row = int(np.ceil(jp2k_tile_num // num_jp2k_tile_cols))
+        jp2k_tile_col = int(np.ceil(jp2k_tile_num % num_jp2k_tile_cols))
+
+        num_tiff_tile_cols = int(np.ceil(self.imagewidth / self.tw))
+
+        # Upper left corner of the jp2k tile
+        ulx = jtw * jp2k_tile_col
+        uly = jth * jp2k_tile_row
+        ul_tiff_tilenum = libtiff.computeTile(self.tiff_fp, ulx, uly, 0, 0)
+        left_tiff_tile_col = int(np.ceil(ulx // self.tw))
+
+        # Upper right corner of the jp2k tile
+        urx = min(ulx + jtw - 1, self.imagewidth - 1)
+        right_tiff_tile_col = int(np.ceil(urx // self.tw))
+
+        # lower left corner
+        llx = ulx
+        lly = min(uly + jth - 1, self.imageheight - 1)
+        ll_tiff_tilenum = libtiff.computeTile(self.tiff_fp, llx, lly, 0, 0)
+        lower_tiff_tile_row = int(np.ceil(ll_tiff_tilenum // num_tiff_tile_cols))  # noqa : E501
+
+        # lower right corner
+        lrx = min(llx + jtw - 1, self.imagewidth - 1)
+        lry = lly
+        lr_tiff_tilenum = libtiff.computeTile(self.tiff_fp, lrx, lry, 0, 0)
+
+        # collect the tiles
+        tiles = set()
+
+        for tile_num in range(ul_tiff_tilenum, lr_tiff_tilenum + 1):
+
+            tiff_tile_col = int(np.ceil(tile_num % num_tiff_tile_cols))
+
+            # The tile rows should always be good.  But we need to exclude
+            # any tiles based on the tile column.
+            if (
+                tiff_tile_col < left_tiff_tile_col
+                or tiff_tile_col > right_tiff_tile_col
+            ):
+                continue
+
+            # otherwise, this is a tile that intersects the jp2k tile
+            tiles.add(tile_num)
+
+        return tiles
+
+    def _write_striped_tiff_to_tiled_jp2k(self):
+        """The input TIFF image is striped and we are to create the output
+        JPEG2000 image as a tiled JP2K.
         """
 
-        self.logger.debug(f'image:  {imageheight} x {imagewidth}')
+        jth, jtw = self.tilesize
+
+        self.logger.debug(f'image:  {self.imageheight} x {self.imagewidth}')
         self.logger.debug(f'jptile:  {jth} x {jtw}')
         num_strips = libtiff.numberOfStrips(self.tiff_fp)
 
-        num_jp2k_tile_cols = int(np.ceil(imagewidth / jtw))
+        num_jp2k_tile_cols = int(np.ceil(self.imagewidth / jtw))
 
         for idx, tilewriter in enumerate(self.jp2.get_tilewriters()):
 
             jp2k_tile_row = idx // num_jp2k_tile_cols
             jp2k_tile_col = idx % num_jp2k_tile_cols
 
-            msg = (
-                f'Tile:  #{idx} '
-                f'row #{jp2k_tile_row} '
-                f'col #{jp2k_tile_col}'
-            )
+            msg = f'Tile:  #{idx} row #{jp2k_tile_row} col #{jp2k_tile_col}'
             self.logger.info(msg)
 
             # the coordinates of the upper left pixel of the jp2k tile
-            julr, julc = jp2k_tile_row * jth, jp2k_tile_col * jtw
+            july, julx = jp2k_tile_row * jth, jp2k_tile_col * jtw
 
+            # If we are starting a new row of jp2k tiles, we want to allocate
+            # space for all the TIFF strips that encompass this jp2k row, and
+            # then go ahead and read them in.  For all other cases, just assign
+            # jp2k tiles from this same TIFF multi-strip.
             if jp2k_tile_col == 0:
-
                 tiff_multi_strip = self._construct_multi_strip(
-                    julr, num_strips, jth, imageheight, imagewidth, rps, spp,
-                    dtype, use_rgba_interface
+                    july, num_strips, jth,
                 )
 
-            # Get the multi-strip coordinates of the upper left jp2k corner
-            stripnum = libtiff.computeStrip(self.tiff_fp, julr, 0)
-            tulr = stripnum * rps
+            # construct the TIFF row and column slices from the multi-strip,
+            # assign to the jp2k tile.
+            stripnum = libtiff.computeStrip(self.tiff_fp, july, 0)
+            tile_uly = stripnum * self.rps
 
-            ms_ulr = julr - tulr
-            ms_ulc = julc
+            ms_uly = july - tile_uly
+            ms_ulx = julx
 
-            # ms_lrr = ms_ulr + min(ms_ulr + jth, imageheight)
-            # ms_lrc = ms_ulc + min(ms_ulc + jtw, imagewidth)
-            ms_lrr = min(ms_ulr + jth, imageheight - tulr)
-            ms_lrc = min(ms_ulc + jtw, imagewidth)
+            ms_lry = min(ms_uly + jth, self.imageheight - tile_uly)
+            ms_lrx = min(ms_ulx + jtw, self.imagewidth)
 
-            rows = slice(ms_ulr, ms_lrr)
-            cols = slice(ms_ulc, ms_lrc)
+            rows = slice(ms_uly, ms_lry)
+            cols = slice(ms_ulx, ms_lrx)
 
             tilewriter[:] = tiff_multi_strip[rows, cols, :]
 
-    def _construct_multi_strip(
-        self, julr, num_strips, jth, imageheight, imagewidth, rps, spp, dtype,
-        use_rgba_interface
-    ):
+    def _construct_multi_strip(self, july, num_strips, jth):
         """TIFF strips are pretty inefficient.  If our I/O was stupidly focused
         solely on each JP2K tile, we would read in each TIFF strip multiple
-        times.  If instead, we read in ALL the strips that will encompass that
-        current row of JP2K tiles, we will save ourselves from pushing around
-        a lot of electrons.
+        times, once for each JP2K tile in the JP2K tile row.  If instead, we
+        read in ALL the strips that will encompass that current row of JP2K
+        tiles, we will save ourselves a lot of disk I/O.
 
         Parameters
         ----------
-        imagewidth, imageheight, spp : int
-            TIFF tag values corresponding to the photometric interpretation,
-            image width and height, and samples per pixel.
         jth : int
             The number of rows in a JP2K tile.
-        rps : int
-            The number of rows per strip in the TIFF.
-        julr : int
+        july : int
             The top row of the current JP2K tile row.
-        dtype : np.dtype
-            Datatype of the image.
-        use_rgba_interface : bool
-            If true, use the RGBA interface to read the TIFF image data.
 
         Returns
         -------
@@ -1033,55 +1085,92 @@ class Tiff2Jp2k(object):
             Holds all the TIFF strips that tightly encompass the current JP2K
             tile row.
         """
+        num_strips = libtiff.numberOfStrips(self.tiff_fp)
+
         # We need to create a TIFF "multi-strip" that can hold all of
         # the JP2K tiles in a JP2K tile row.
-        r = julr
-        top_strip_num = libtiff.computeStrip(self.tiff_fp, r, 0)
+        y = july
+        top_strip_num = libtiff.computeStrip(self.tiff_fp, y, 0)
 
-        # advance thru to the next tile row
-        while (r // rps) * rps < min(julr + jth, imageheight):
-            r += rps
-        bottom_strip_num = libtiff.computeStrip(self.tiff_fp, r, 0)
+        # Find the strip number that is ONE MORE than the bottom strip.
+        while (y // self.rps) * self.rps < min(july + jth, self.imageheight):
+            y += self.rps
+        bottom_strip_num = libtiff.computeStrip(self.tiff_fp, y, 0)
 
         # compute the number of rows contained between the top strip
         # and the bottom strip
-        num_rows = (bottom_strip_num - top_strip_num) * rps
+        num_rows = (bottom_strip_num - top_strip_num) * self.rps
 
-        if use_rgba_interface:
-            tiff_strip = np.zeros((rps, imagewidth, 4), dtype=np.uint8)
-            tiff_multi_strip = np.zeros(
-                (num_rows, imagewidth, spp), dtype=np.uint8
-            )
+        if self.photo == libtiff.Photometric.YCBCR:
+            # always single byte samples of R, G, B, and A
+            dtype = np.uint8
         else:
-            tiff_strip = np.zeros((rps, imagewidth, spp), dtype=dtype)
-            tiff_multi_strip = np.zeros(
-                (num_rows, imagewidth, spp), dtype=dtype
-            )
+            dtype = self.dtype
+        spp = self.spp
 
-        # fill the multi-strip
+        # This may result in a multi-strip that has more rows than the jp2k
+        # tile
+        tiff_multi_strip = np.zeros(
+            (num_rows, self.imagewidth, spp), dtype=dtype
+        )
+
+        # Fill the multi-strip
         for stripnum in range(top_strip_num, bottom_strip_num):
-            tiff_strip = np.zeros((rps, imagewidth, spp), dtype=dtype)
 
-            if use_rgba_interface:
+            if self.photo == libtiff.Photometric.YCBCR:
 
-                libtiff.readRGBAStrip(
-                    self.tiff_fp, stripnum * rps, tiff_strip
+                tiff_rgba_strip = np.zeros(
+                    (self.rps, self.imagewidth, 4), dtype=dtype
                 )
 
-                # must flip the rows (!!) and get rid of the alpha
-                # plane
-                tiff_strip = np.flipud(tiff_strip[:, :, :spp])
+                libtiff.readRGBAStrip(
+                    self.tiff_fp, stripnum * self.rps, tiff_rgba_strip
+                )
+
+                # If a partial last strip...
+                if (
+                    stripnum == num_strips - 1
+                    and self.imageheight // self.rps != num_strips
+                ):
+                    # According to the man page:
+                    #
+                    # When reading a partial last strip in the file the last
+                    # line of the image  will  begin at the beginning of the
+                    # buffer.
+                    #
+                    # move the top strips down to the bottom, otherwise the
+                    # following flipping logic doesn't work.
+                    bottom_row = self.imageheight % self.rps
+                    irows = slice(0, bottom_row)
+                    orows = slice(self.rps - bottom_row, self.rps)
+                    tiff_rgba_strip[orows, :, :] = tiff_rgba_strip[irows, :, :]
+
+                # The rgba interface requires at least flipping the image
+                # upside down, and also reordering the planes on big endian
+                if sys.byteorder == 'little':
+                    dims = [0]
+                else:
+                    dims = [0, 2]
+                tiff_rgba_strip = np.flip(tiff_rgba_strip, axis=dims)
+
+                # potentially get rid of alpha plane
+                tiff_strip = tiff_rgba_strip[:, :, :self.spp]
 
             else:
 
+                tiff_strip = np.zeros(
+                   (self.rps, self.imagewidth, spp), dtype=dtype
+                )
+
                 libtiff.readEncodedStrip(
                     self.tiff_fp, stripnum, tiff_strip
                 )
 
             # push the strip into the multi-strip
-            top_row = (stripnum - top_strip_num) * rps
-            bottom_row = (stripnum - top_strip_num + 1) * rps
+            top_row = (stripnum - top_strip_num) * self.rps
+            bottom_row = (stripnum - top_strip_num + 1) * self.rps
             rows = slice(top_row, bottom_row)
+
             tiff_multi_strip[rows, :, :] = tiff_strip
 
         return tiff_multi_strip
