@@ -2,9 +2,11 @@
 """
 # Standard library imports ...
 import doctest
+import importlib.resources as ir
 from io import BytesIO
 import os
 import pathlib
+import pickle
 import re
 import shutil
 import struct
@@ -28,7 +30,6 @@ from glymur.jp2box import (
 from glymur.core import COLOR, OPACITY, SRGB, GREYSCALE
 from glymur.core import RED, GREEN, BLUE, GREY, WHOLE_IMAGE
 from . import fixtures
-from .fixtures import MetadataBase
 from .fixtures import OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG
 
 
@@ -143,7 +144,7 @@ class TestChannelDefinition(fixtures.TestCommon):
         shutil.rmtree(cls.planes_dir)
 
     def setUp(self):
-        super(TestChannelDefinition, self).setUp()
+        super().setUp()
 
         j2k = Jp2k(self.j2kfile)
         codestream = j2k.get_codestream()
@@ -376,7 +377,7 @@ class TestFileTypeBox(fixtures.TestCommon):
 
         EXPECTED RESULT:  RuntimeError
         """
-        path = fixtures._path_to('issue396.jp2')
+        path = ir.files('tests.data').joinpath('issue396.jp2')
         with warnings.catch_warnings():
             # Lots of things wrong with this file.
             warnings.simplefilter('ignore')
@@ -439,6 +440,28 @@ class TestResolutionBoxes(fixtures.TestCommon):
         self.assertEqual(newbox.box[1].vertical_resolution, 2.5)
         self.assertEqual(newbox.box[1].horizontal_resolution, 0.5)
 
+    def test_pickling(self):
+        """
+        SCENARIO:  pickle a ResolutionBox
+
+        Expected Results:  should not error out
+        """
+        vres = 0.5
+        hres = 2.5
+        resc = glymur.jp2box.CaptureResolutionBox(vres, hres)
+        resd = glymur.jp2box.DisplayResolutionBox(vres, hres)
+        rbox = glymur.jp2box.ResolutionBox(box=[resc, resd])
+
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            rbox.write(tfile)
+
+        with open(self.temp_jp2_filename, mode='rb') as tfile:
+            tfile.seek(8)
+            rbox_read = glymur.jp2box.ResolutionBox.parse(tfile, 0, 44)
+
+            with tempfile.NamedTemporaryFile(mode='wb') as t2:
+                pickle.dump(rbox_read, t2)
+
     def test_resolution_superbox(self):
         """
         SCENARIO:  write a resolution superbox
@@ -1175,8 +1198,21 @@ class TestJp2Boxes(fixtures.TestCommon):
         )
 
 
-class TestRepr(MetadataBase):
+class TestRepr(fixtures.TestCommon):
     """Tests for __repr__ methods."""
+
+    def _verify_filetype_box(self, actual, expected):
+        """
+        All JP2 files should have a brand reading 'jp2 ' and just a single
+        entry in the compatibility list, also 'jp2 '.  JPX files can have more
+        compatibility items.
+        """
+        self.assertEqual(actual.brand, expected.brand)
+        self.assertEqual(actual.minor_version, expected.minor_version)
+        self.assertEqual(actual.minor_version, 0)
+        for cl in expected.compatibility_list:
+            self.assertIn(cl, actual.compatibility_list)
+
     def test_default_jp2k(self):
         """Should be able to eval a JPEG2000SignatureBox"""
         jp2k = glymur.jp2box.JPEG2000SignatureBox()
@@ -1265,7 +1301,7 @@ class TestRepr(MetadataBase):
 
         # Test the representation instantiation.
         newbox = eval(repr(ftyp))
-        self.verify_filetype_box(newbox, FileTypeBox())
+        self._verify_filetype_box(newbox, FileTypeBox())
 
     def test_colourspecification_box(self):
         """Verify __repr__ method on colr box."""
