@@ -1067,7 +1067,7 @@ class TestPrinting(fixtures.TestCommon):
                 "                    (   'TileOffsets',\n"
                 "                        "
                 "array([ 0, 10, 20, ..., 70, 80, 90], dtype=uint32)),\n"
-                "                    ('ExifTag', OrderedDict({'Make': 'HTC'}))])"
+                "                    ('ExifTag', OrderedDict({'Make': 'HTC'}))])"  # noqa : E501
             )
         else:
             expected = (
@@ -1078,7 +1078,7 @@ class TestPrinting(fixtures.TestCommon):
                 "                    (   'TileOffsets',\n"
                 "                        "
                 "array([ 0, 10, 20, ..., 70, 80, 90], dtype=uint32)),\n"
-                "                    ('ExifTag', OrderedDict([('Make', 'HTC')]))])"
+                "                    ('ExifTag', OrderedDict([('Make', 'HTC')]))])"  # noqa : E501
             )
         self.assertEqual(actual, expected)
 
@@ -1829,8 +1829,7 @@ class TestJp2dump(unittest.TestCase):
             "    y1: 0\n"
             "    numcomps: 0\n"
             "    color_space: 0\n"
-            "    icc_profile_buf: "
-            "<glymur.lib.openjp2.LP_c_ubyte object at 0x[0-9A-Fa-f]*>\n"
+            "    icc_profile_buf: <(glymur.lib.openjp2|ctypes.wintypes).LP_c_ubyte object at 0x[0-9A-Fa-f]*>\n"  # noqa : E501
             "    icc_profile_len: 0")
         self.assertRegex(actual, expected)
 
